/*
	$Id: treeview.h,v 1.11 2001/10/10 12:17:10 sphair Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanGUI="Controls"
//! header=gui.h

#ifndef header_treeview
#define header_treeview

#include "component.h"

class CL_TreeView_Generic;

//: TreeView node
class CL_TreeView_Node
{
//! Construction:
public:
	CL_TreeView_Node();

//! Attributes:
public:
	//: Returns the number of children.
	int get_count() const;

	//: Returns the item at position index.
	CL_Component *get_item(int index) const;

	//: Returns the index of the current item, or -1 if no item has been selected. 
	int get_current_item() const;

	//: Returns true if item index is selected. Returns false if it is not selected or if there is an error.
	bool is_selected(int index) const;

//! Operations:
public:
	//: Inserts an item at index. 
	//- If index is negative, text is inserted at the end of the list.
	//- Returns the node of the item.
	CL_TreeView_Node *insert_item(const std::string, int index = -1);

	//: Inserts an item (uses any type of component) at index. 
	//- If index is negative, text is inserted at the end of the list.
	//- Returns the node of the item.
	CL_TreeView_Node *insert_item(CL_Component *component, int index = -1);

	//: Removes and deletes the item at position index.
	//: If index is equal to get_current_item(), a new item gets highlighted and the sig_highlighted() signal is emitted. 
	void remove_item(int index);

	//: Sets the highlighted item to the item index.
	//: The highlighting is moved and the treeview scrolled as necessary. 
	void set_current_item(int index);

	//: Selects item at index if select is true, or unselects it if select is false.
	//: If the treeview is a single-selection listbox and and select is true, set_current_item() is called. 
	//: If the treeview is a single-selection listbox and and select is false, clear_selection()
	//: is called if index is the currently selected item. 
	void set_selected(int index, bool select);

	//: Deselects current selected item.
	void clear_selection();

	//: If select is true, all items get selected, else all get unselected.
	//: This works only in Multi selection mode.
	void select_all(bool select = true);

	//: Inverts the selection.
	//: Works only in Multi selection mode.
	void invert_selection();

	//: Sets the listbox to multiselection if enable is true,
	//: single selection otherwise.
	void set_multi_selection(bool enable = true);

	//: Deletes all items in the list. 
	void clear();

	void draw_nodes(CL_Point &point);

//! Implementation:
private:
	void draw_node(CL_Point &point);
	void on_child_click(const CL_Key &key);

	CL_Component *component;
	std::list<CL_TreeView_Node *> children;
	std::list<CL_Slot *> slots_child_click;

	bool collapsed;
};

//: TreeView component
class CL_TreeView : public CL_Component, public CL_TreeView_Node
{
//! Construction:
public:
	//: TreeView Constructor
	CL_TreeView(
		const CL_ComponentOptions &options,
		CL_Component *parent,
		CL_StyleManager *style = NULL);
	
	//: TreeView Constructor
	CL_TreeView(
		const CL_Rect &pos,
		CL_Component *parent,
		CL_StyleManager *style = NULL);

	//: TreeView Destructor
	virtual ~CL_TreeView();

//! Attributes:
public:
	//: Returns the client area of the component.
	CL_Component *get_client_area() const;

//! Operations:
public:

//! Signals:
public:

//! Implementation:
private:
	CL_TreeView(const CL_TreeView &copy) : CL_Component(NULL, NULL) { return; } // disallow copy construction.
	CL_TreeView_Generic *impl;
};

#endif
